"Change Set:		7669TrueType-ar.8

TrueType-ar.8:
Process typographic metrics from the OS/2 table in truetype fonts.

"!
Object subclass: #TTFileDescription
	instanceVariableNames: 'fileName fileOffset familyName subfamilyName ascender descender lineGap unitsPerEm numGlyphs indexToLocOffset indexToLocFormat glyphTableOffset cmapType cmapOffset numHMetrics hmtxTableOffset sTypoAscender sTypoDescender sTypoLineGap'
	classVariableNames: 'AllFontsAndFiles FontPaths OfferNonPortableFonts'
	poolDictionaries: ''
	category: 'TrueType-Fonts'!

Object subclass: #TTFontDescription
	instanceVariableNames: 'glyphTable glyphs kernPairs copyright familyName fullName subfamilyName uniqueName versionName postscriptName trademark bounds unitsPerEm ascender descender lineGap sTypoAscender sTypoDescender sTypoLineGap'
	classVariableNames: 'Default Descriptions'
	poolDictionaries: ''
	category: 'TrueType-Fonts'!

!TTFileDescription methodsFor: 'rendering' stamp: 'ar 8/29/2009 19:11'!
renderGlyph: code height: height fgColor: fgColor bgColor: bgColor depth: depth
	"Render the glyph with the given code point at the specified pixel height."
	^(self at: code) 
		asFormWithScale: height asFloat / (ascender - descender) 
			ascender: ascender 
			descender: descender 
			fgColor: fgColor bgColor: bgColor depth: depth! !

!TTFileDescription methodsFor: 'accessing' stamp: 'ar 8/29/2009 19:08'!
descender
	"Descender of the font. Relative to unitsPerEm.
	Easily confused with the typographic descender."
	^descender! !

!TTFileDescription methodsFor: 'accessing' stamp: 'ar 8/29/2009 19:07'!
ascender
	"Ascender of the font. Relative to unitsPerEm.
	Easily confused with the typographic ascender."
	^ascender! !

!TTFileDescription methodsFor: 'accessing' stamp: 'ar 8/29/2009 19:10'!
typographicAscender
	"Microsoft defines this as the 'true typographic metrics' of the font."
	^sTypoAscender ifNil:[ascender]! !

!TTFontDescription methodsFor: 'accessing' stamp: 'ar 8/29/2009 19:10'!
typographicDescender
	"Microsoft defines this as the 'true typographic metrics' of the font."
	^sTypoDescender ifNil:[descender]! !

!TTFileDescription methodsFor: 'accessing' stamp: 'ar 8/29/2009 19:08'!
lineGap
	"Leading of the font. Relative to unitsPerEm.
	Easily confused with the typographic linegap."
	^lineGap! !

!TTFontDescription methodsFor: 'accessing' stamp: 'ar 8/29/2009 19:10'!
typographicLineGap
	"Microsoft defines this as the 'true typographic metrics' of the font."
	^sTypoLineGap ifNil:[lineGap]! !

!TTFontDescription methodsFor: 'rendering' stamp: 'ar 8/29/2009 19:12'!
renderGlyph: code height: fontHeight fgColor: fgColor bgColor: bgColor depth: depth
	"Render the glyph with the given code point at the specified pixel height."
	^(self at: code) 
		asFormWithScale: fontHeight asFloat / (ascender - descender) 
			ascender: ascender 
			descender: descender 
			fgColor: fgColor bgColor: bgColor depth: depth! !

!TTFontDescription methodsFor: 'properties' stamp: 'ar 8/29/2009 19:11'!
descender
	"Descender of the font. Relative to unitsPerEm.
	Easily confused with the typographic descender."
	^descender! !

!TTFileDescription methodsFor: 'accessing' stamp: 'ar 8/29/2009 18:46'!
fontHeight
	^ascender - descender! !

!TTFontDescription methodsFor: 'properties' stamp: 'ar 8/29/2009 19:10'!
ascender
	"Ascender of the font. Relative to unitsPerEm.
	Easily confused with the typographic ascender."
	^ascender! !

!TTFileDescription methodsFor: 'initialize' stamp: 'ar 8/29/2009 11:45'!
on: aFileName offset: fontOffset
	"Initialize the receiver from a file name"
	fileName := aFileName.
	fileOffset := fontOffset.
	self withFileDo:[:fontFile|
		"Some bitmap fonts are called .ttf; skip anything that doesn't have a header"
		(self findTable: 'head' in: fontFile) ifFalse:[^nil].
		self processFontHeaderTable: fontFile.
		(self findTable: 'maxp' in: fontFile) 
			ifFalse:[^self error: 'File does not have a profile table'].
		self processMaximumProfileTable: fontFile.
		(self findTable: 'name' in: fontFile) 
			ifFalse:[^self error: 'File does not have a naming table'].
		self processNamingTable: fontFile.
		(self findTable: 'hhea' in: fontFile) 
			ifFalse:[^self error: 'File does not have a horizontal header table'].
		self processHorizontalHeaderTable: fontFile.
		(self findTable: 'OS/2' in: fontFile)
			ifTrue:[self processOS2Table: fontFile].
		(self findTable: 'hmtx' in: fontFile) 
			ifFalse:[^self error: 'File does not have a horizontal header table'].
		hmtxTableOffset := fontFile position.
		(self findTable: 'loca' in: fontFile) 
			ifFalse:[^self error: 'File does not have a naming table'].
		indexToLocOffset := fontFile position.
		(self findTable: 'glyf' in: fontFile) 
			ifFalse:[^self error: 'File does not have a naming table'].
		glyphTableOffset := fontFile position.
		(self findTable: 'cmap' in: fontFile) 
			ifFalse:[^self error: 'File does not have a header table'].
		self processCharacterMappingTable: fontFile.
	].! !

!TTFontDescription methodsFor: 'accessing' stamp: 'ar 8/29/2009 19:10'!
typographicAscender
	"Microsoft defines this as the 'true typographic metrics' of the font."
	^sTypoAscender ifNil:[ascender]! !

!TTFontDescription methodsFor: 'private-initialization' stamp: 'ar 8/29/2009 19:19'!
setTypographicAscender: asc descender: desc lineGap: lGap
	sTypoAscender := asc.
	sTypoDescender := desc.
	sTypoLineGap := lGap.
! !

!TTFontDescription methodsFor: 'properties' stamp: 'ar 8/29/2009 19:11'!
lineGap
	"Leading of the font. Relative to unitsPerEm.
	Easily confused with the typographic linegap."
	^lineGap! !

!TTFileDescription methodsFor: 'accessing' stamp: 'ar 8/29/2009 19:10'!
typographicDescender
	"Microsoft defines this as the 'true typographic metrics' of the font."
	^sTypoDescender ifNil:[descender]! !

!TTFontDescription methodsFor: 'accessing' stamp: 'ar 8/29/2009 18:46'!
fontHeight
	^ascender - descender! !

!TTFileDescription methodsFor: 'ttf tables' stamp: 'ar 8/29/2009 11:53'!
processOS2Table: fontFile
"
	USHORT  	 version   	0x0004
	SHORT 	xAvgCharWidth 	 
	USHORT 	usWeightClass 	 
	USHORT 	usWidthClass 	 
	USHORT 	fsType 	 
	SHORT 	ySubscriptXSize 	 
	SHORT 	ySubscriptYSize 	 
	SHORT 	ySubscriptXOffset 	 
	SHORT 	ySubscriptYOffset 	 
	SHORT 	ySuperscriptXSize 	 
	SHORT 	ySuperscriptYSize 	 
	SHORT 	ySuperscriptXOffset 	 
	SHORT 	ySuperscriptYOffset 	 
	SHORT 	yStrikeoutSize 	 
	SHORT 	yStrikeoutPosition 	 
	SHORT 	sFamilyClass 	 
	BYTE 	panose[10] 	 
	ULONG 	ulUnicodeRange1 	Bits 0-31
	ULONG 	ulUnicodeRange2 	Bits 32-63
	ULONG 	ulUnicodeRange3 	Bits 64-95
	ULONG 	ulUnicodeRange4 	Bits 96-127
	CHAR 	achVendID[4] 	 
	USHORT 	fsSelection 	 
	USHORT 	usFirstCharIndex 	 
	USHORT 	usLastCharIndex 	 
	SHORT 	sTypoAscender 	 
	SHORT 	sTypoDescender 	 
	SHORT 	sTypoLineGap 	 
	USHORT 	usWinAscent 	 
	USHORT 	usWinDescent 	 
	ULONG 	ulCodePageRange1 	Bits 0-31
	ULONG 	ulCodePageRange2 	Bits 32-63
	SHORT 	sxHeight 	 
	SHORT 	sCapHeight 	 
	USHORT 	usDefaultChar 	 
	USHORT 	usBreakChar 	 
	USHORT 	usMaxContext 	 "
	| version fsSelection minAscii maxAscii |
	version := self short: (fontFile nextNumber: 2). "table version"
	version = 0 ifTrue:[^self].
	fontFile skip: 60.
	fsSelection := fontFile nextNumber: 2.
	minAscii := fontFile nextNumber: 2.
	maxAscii := fontFile nextNumber: 2.
	sTypoAscender := self short: (fontFile nextNumber: 2).
	sTypoDescender := self short: (fontFile nextNumber: 2).
	sTypoLineGap := self short: (fontFile nextNumber: 2).
! !

!TTFontReader methodsFor: 'processing' stamp: 'ar 8/29/2009 19:19'!
processOS2Table: entry
"
	USHORT  	 version   	0x0004
	SHORT 	xAvgCharWidth 	 
	USHORT 	usWeightClass 	 
	USHORT 	usWidthClass 	 
	USHORT 	fsType 	 
	SHORT 	ySubscriptXSize 	 
	SHORT 	ySubscriptYSize 	 
	SHORT 	ySubscriptXOffset 	 
	SHORT 	ySubscriptYOffset 	 
	SHORT 	ySuperscriptXSize 	 
	SHORT 	ySuperscriptYSize 	 
	SHORT 	ySuperscriptXOffset 	 
	SHORT 	ySuperscriptYOffset 	 
	SHORT 	yStrikeoutSize 	 
	SHORT 	yStrikeoutPosition 	 
	SHORT 	sFamilyClass 	 
	BYTE 	panose[10] 	 
	ULONG 	ulUnicodeRange1 	Bits 0-31
	ULONG 	ulUnicodeRange2 	Bits 32-63
	ULONG 	ulUnicodeRange3 	Bits 64-95
	ULONG 	ulUnicodeRange4 	Bits 96-127
	CHAR 	achVendID[4] 	 
	USHORT 	fsSelection 	 
	USHORT 	usFirstCharIndex 	 
	USHORT 	usLastCharIndex 	 
	SHORT 	sTypoAscender 	 
	SHORT 	sTypoDescender 	 
	SHORT 	sTypoLineGap 	 
	USHORT 	usWinAscent 	 
	USHORT 	usWinDescent 	 
	ULONG 	ulCodePageRange1 	Bits 0-31
	ULONG 	ulCodePageRange2 	Bits 32-63
	SHORT 	sxHeight 	 
	SHORT 	sCapHeight 	 
	USHORT 	usDefaultChar 	 
	USHORT 	usBreakChar 	 
	USHORT 	usMaxContext 	 "
	| version fsSelection minAscii maxAscii asc desc lGap |
	version := entry nextShort. "table version"
	version = 0 ifTrue:[^self].
	entry skip: 60.
	fsSelection := entry nextUShort.
	minAscii := entry nextUShort.
	maxAscii := entry nextUShort.
	asc := entry nextShort.
	desc := entry nextShort.
	lGap := entry nextShort.
	fontDescription setTypographicAscender: asc descender: desc lineGap: lGap.! !

!TTFileDescription methodsFor: 'accessing' stamp: 'ar 8/29/2009 19:10'!
typographicLineGap
	"Microsoft defines this as the 'true typographic metrics' of the font."
	^sTypoLineGap ifNil:[lineGap]! !

