"Change Set:		8668Monticello-ar.347

Monticello-ar.347:
Have MCPackageLoader use the best guess about the package being installed in progress notifications.

Monticello-ar.346:
Include names of repository and package in up and download progress.

"!
!MCPackageLoader methodsFor: 'private' stamp: 'ar 12/30/2009 17:24'!
basicLoad
	"Load the contents of some package. This is the core loading method
	in Monticello. Be wary about modifying it unless you understand the details
	and dependencies of the various entities being modified."
	| pkgName |
	errorDefinitions := OrderedCollection new.
	"Obviously this isn't the package name but we don't have anything else
	to use here. ChangeSet current name will generally work since a CS is 
	usually installed prior to installation."
	pkgName := ChangeSet current name.

	[["Pass 1: Load everything but the methods,  which are collected in methodAdditions."
	additions do: [:ea | 
		[ea isMethodDefinition 
			ifTrue:[methodAdditions add: ea asMethodAddition]
			ifFalse:[ea load]]on: Error do: [errorDefinitions add: ea].
	] displayingProgress: 'Reshaping ', pkgName.

	"Pass 2: We compile new / changed methods"
	methodAdditions do:[:ea| ea createCompiledMethod] 
		displayingProgress: 'Compiling ', pkgName.

	'Installing ', pkgName displayProgressAt: Sensor cursorPoint from: 0 to: 2 during:[:bar|
		"There is no progress *during* installation since a progress bar update
		will redraw the world and potentially call methods that we're just trying to install."
		bar value: 1.

		"Pass 3: Install the new / changed methods
		(this is a separate pass to allow compiler changes to be loaded)"
		methodAdditions do:[:ea| ea installMethod].

		"Pass 4: Remove the obsolete methods"
		removals do:[:ea| ea unload].
	].

	"Try again any delayed definitions"
	self shouldWarnAboutErrors ifTrue: [self warnAboutErrors].
	errorDefinitions do: [:ea | ea load] 
		displayingProgress: 'Reloading ', pkgName.

	"Finally, notify observers for the method additions"
	methodAdditions do: [:each | each notifyObservers] 
		"the message is fake but actually telling people how much time we spend
		in the notifications is embarrassing so lie instead"
		displayingProgress: 'Installing ', pkgName.

	additions do: [:ea | ea postloadOver: (self obsoletionFor: ea)] 
		displayingProgress: 'Initializing ', pkgName.

	] on: InMidstOfFileinNotification do: [:n | n resume: true]
	] ensure: [self flushChangesFile]! !

!MCHttpRepository methodsFor: 'required' stamp: 'ar 12/30/2009 17:02'!
allFileNames
	| index |
	self displayProgress: 'Updating ', self description during:[
		index := HTTPSocket httpGet: self locationWithTrailingSlash, '?C=M;O=D' args: nil user: self user passwd: self password.
	].
	index isString ifTrue: [self error: 'Could not access ', location].
	^ self parseFileNamesFromStream: index	! !

!MCHttpRepository methodsFor: 'required' stamp: 'ar 12/30/2009 17:02'!
readStreamForFileNamed: aString do: aBlock
	| contents |
	self displayProgress: 'Downloading ', aString during:[
		contents := HTTPSocket httpGet: (self urlForFileNamed: aString) args: nil user: self user passwd: self password.
	].
	^ contents isString ifFalse: [aBlock value: contents]! !

!MCHttpRepository methodsFor: 'required' stamp: 'ar 12/30/2009 17:02'!
writeStreamForFileNamed: aString replace: ignoreBoolean do: aBlock
	| stream response |
	stream := RWBinaryOrTextStream on: String new.
	aBlock value: stream.
	self displayProgress: 'Uploading ', aString during:[
		response := HTTPSocket
					httpPut: stream contents
					to: (self urlForFileNamed: aString)
					user: self user
					passwd: self password.
	].
	(#( 'HTTP/1.1 201 ' 'HTTP/1.1 200 ' 'HTTP/1.0 201 ' 'HTTP/1.0 200 ')
		anySatisfy: [:code | response beginsWith: code ])
			ifFalse: [self error: response].! !

